class Webui::SessionController < Webui::WebuiController
  before_action :kerberos_auth, only: [:new]

  skip_before_action :check_anonymous, only: [:create]

  def new
    switch_to_webui2
  end

  def create
    user = User.find_with_credentials(params[:username], params[:password])

    unless user
      RabbitmqBus.send_to_bus('metrics', 'login,access_point=webui,failure=unauthenticated value=1')
      redirect_to(session_new_path, error: 'Authentication failed')
      return
    end

    unless user.is_active?
      RabbitmqBus.send_to_bus('metrics', 'login,access_point=webui,failure=disabled value=1')
      redirect_to(root_path, error: 'Your account is disabled. Please contact the administrator for details.')
      return
    end

    User.session = user
    session[:login] = user.login
    Rails.logger.debug "Authenticated as user '#{user.login}'"
    RabbitmqBus.send_to_bus('metrics', 'login,access_point=webui value=1')

    redirect_on_login
  end

  def destroy
    Rails.logger.info "Logging out: #{session[:login]}"

    reset_session
    RabbitmqBus.send_to_bus('metrics', 'logout,access_point=webui value=1')
    User.session = nil

    redirect_on_logout
  end

  private

  def redirect_on_login
    if referer_was_login?
      redirect_to user_show_path(User.session!)
    else
      redirect_back(fallback_location: root_path)
    end
  end

  def redirect_on_logout
    if CONFIG['proxy_auth_mode'] == :on
      redirect_to CONFIG['proxy_auth_logout_page']
    else
      redirect_back(fallback_location: root_path)
    end
  end

  def referer_was_login?
    request.referer && request.referer.end_with?(session_new_path)
  end
end
