module Event
  class ReviewChanged < Request
    self.message_bus_routing_key = 'request.review_changed'
    self.description = 'Request was reviewed'
    payload_keys :reviewers, :by_user, :by_group, :by_project, :by_package
    receiver_roles :source_maintainer, :target_maintainer, :creator, :source_watcher, :target_watcher

    def subject
      "Request #{payload['number']} was reviewed (#{actions_summary})"
    end

    def expanded_payload
      payload_with_diff
    end

    def custom_headers
      super.merge(review_headers)
    end
  end
end

# == Schema Information
#
# Table name: events
#
#  id          :integer          not null, primary key
#  eventtype   :string(255)      not null, indexed
#  payload     :text(65535)
#  created_at  :datetime         indexed
#  updated_at  :datetime
#  undone_jobs :integer          default(0)
#  mails_sent  :boolean          default(FALSE), indexed
#
# Indexes
#
#  index_events_on_created_at  (created_at)
#  index_events_on_eventtype   (eventtype)
#  index_events_on_mails_sent  (mails_sent)
#
